/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.frameworks.jcr;

import java.util.HashMap;
import java.util.Map;

import javax.jcr.Credentials;
import javax.jcr.LoginException;
import javax.jcr.NoSuchWorkspaceException;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;

import org.exoplatform.services.jcr.core.ManageableRepository;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: SingleRepositorySessionFactory.java 5800 2006-05-28 18:03:31Z geaz $
 */

public class SingleRepositorySessionFactory implements JCRAppSessionFactory {

  public static final String SESSION_FACTORY = "org.exoplatform.frameworks.web.sessionFactory";

  private Map <String, Session> cache;
  private Repository rep;
  private Credentials user;
  private String defaultWorkspace;
  
  
  public SingleRepositorySessionFactory(ManageableRepository repository, Credentials cred) {
    cache = new HashMap<String, Session>();
    rep = repository;
    user = cred;
    defaultWorkspace = repository.getConfiguration().getDefaultWorkspaceName();
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.frameworks.jcr.command.JCRSessionFactory#getSession(java.lang.String)
   */
  public Session getSession(String workspaceName) throws LoginException, NoSuchWorkspaceException, RepositoryException {
    
    if (workspaceName == null || workspaceName.length() == 0) {
      if (defaultWorkspace == null)
        throw new NoSuchWorkspaceException("No workspace found");
      else
        workspaceName = defaultWorkspace;
    }

    Session ses = cache.get(workspaceName);
    // create and cache new session 
    if (ses == null) {
      try {
        // if already authenticated 
        ses = rep.login(workspaceName);
      } catch (LoginException e) {
        // use some defaults (anonim)
        ses = rep.login(user, workspaceName);
      }
      cache.put(workspaceName, ses);
    }
    return ses;
  }
}
