/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.frameworks.jcr.command.web;

import java.io.InputStream;
import java.io.PrintWriter;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.chain.Command;
import org.apache.commons.chain.Context;
import org.exoplatform.frameworks.jcr.command.DefaultKeys;
/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: DisplayResourceCommand.java 5800 2006-05-28 18:03:31Z geaz $
 */

public class DisplayResourceCommand implements Command {

  public static String DEFAULT_MAPPING = "/jcr";
  
  private String pathKey = DefaultKeys.PATH;
  private String resultKey = DefaultKeys.RESULT;


  public boolean execute(Context context) throws Exception {
    GenericWebAppContext webCtx = (GenericWebAppContext)context;
    HttpServletResponse response = webCtx.getResponse();
    HttpServletRequest request = webCtx.getRequest();
    Node file = (Node)webCtx.getSession().getItem((String)context.get(pathKey));
      
    Node content;
    if(file.isNodeType("nt:file")) {
      content = file.getNode("jcr:content");
    } else if(file.isNodeType("exo:article")) {
      content = file.getNode("exo:image");
    } else     
      throw new Exception("Invalid node type, expected nt:file or exo:article, have "+file.getPrimaryNodeType().getName()+" at "+file.getPath());

    Property data = content.getProperty("jcr:data");
    String mime = content.getProperty("jcr:mimeType").getString();

    // standalone request?
    String servletPath = request.getPathInfo();
    boolean doClose = true;
    // or included?
    if(servletPath == null) {
      servletPath = (String)request.getAttribute("javax.servlet.include.path_info");
      if(servletPath != null)
        doClose = false;
    }
    
    response.setContentType(mime+"; charset=UTF-8");
    response.setHeader("Cache-Control","no-cache");
    
    //System.out.println("MIME:  > "+mime+" "+servletPath);

    if(mime.startsWith("text")) {
      PrintWriter out = response.getWriter();
      
      out.write(data.getString());
      out.flush();
      if(doClose)
        out.close();
    } else  if(mime.startsWith("image") || mime.startsWith("application")) {
      
      InputStream is = data.getStream();

      byte[] buf = new byte[is.available()];
      is.read(buf);

      ServletOutputStream os = response.getOutputStream();
      os.write(buf);
      os.flush();
      if(doClose)
        os.close();
    }
    
    return true;
  }

//  public String getIncomingNodeKey() {
//    return incomingNodeKey;
//  }

  public String getPathKey() {
    return pathKey;
  }

  public String getResultKey() {
    return resultKey;
  }

}
