/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl;

import java.util.Comparator;

import javax.jcr.RepositoryException;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;

import org.exoplatform.container.ExoContainer;
import org.exoplatform.container.jmx.MX4JComponentAdapterFactory;
import org.exoplatform.services.jcr.access.AccessControlPolicy;
import org.exoplatform.services.jcr.config.RepositoryConfigurationException;
import org.exoplatform.services.jcr.config.RepositoryEntry;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.impl.core.RepositoryImpl;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: RepositoryContainer.java 5834 2006-05-29 12:05:59Z peterit $
 */

public class RepositoryContainer extends ExoContainer {

  private String name;
  
  private MBeanServer mbeanServer;

  public RepositoryContainer(ExoContainer parent, RepositoryEntry config)
      throws RepositoryException, RepositoryConfigurationException {

    //super(parent);
    super(new MX4JComponentAdapterFactory(), parent);

    // Defaults:
    if(config.getAccessControl() == null)
      config.setAccessControl(AccessControlPolicy.OPTIONAL);

    registerComponentInstance(config);

    this.name = config.getName();
    this.mbeanServer = MBeanServerFactory.createMBeanServer("jcrrep" + name + "mx");

    RepositoryInitializer initializer = new RepositoryInitializer(this, config);
    registerComponentInstance(initializer);
    
    // Repository
    RepositoryImpl repository = new RepositoryImpl(this);
    registerComponentInstance(repository);
    
    initializer.init(repository);
  }

  public MBeanServer getMBeanServer() {
    return this.mbeanServer;
  }

  /**
   * @return Returns the name.
   */
  public String getName() {
    return name;
  }
  
  private static class WorkspaceOrderComparator implements Comparator {
    private String sysWs;
    
    private WorkspaceOrderComparator(String sysWs) {
      this.sysWs = sysWs;
    }
    
    public int compare(Object o1, Object o2) {
      String n1 = ((WorkspaceEntry) o1).getName();
      return n1.equals(sysWs)?-1:0;
    }
  }

}