/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.core;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.jcr.RepositoryException;

import org.apache.commons.logging.Log;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL        .
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov</a>
 * @version $Id: ChangedItemsStorage.java 5150 2006-04-17 06:44:21Z geaz $
 */

public class ChangedItemsStorage {

  // uuid -> ItemImpl
  private List itemChanges;
  
  private static Log log = ExoLogger.getLogger("jcr.ChangedItemsStorage");
  
  public ChangedItemsStorage() {
    this.itemChanges = new ArrayList();
  }
  
  /**
   * Adds item to session storage
   * @param item
   */
  synchronized void put(ItemImpl item) {
    log.debug("put "+item.getPath()+" "+item.getInternalUUID()+" "+item.getState());
    itemChanges.add(item);
  }
  
  /**
   * Removes item from session storage
   * @param item
   */
  synchronized void remove(ItemImpl item) {    
    while (itemChanges.contains(item)) { // && maxIter>=0
      itemChanges.remove(item);
    }
  }
  

  /**
   * @param uuid
   * @return item by uuid or null
   */
  synchronized ItemImpl get(String uuid) {
    for(Iterator it=itemChanges.iterator(); it.hasNext();) {
      ItemImpl item = (ItemImpl)it.next();
      if(item.getInternalUUID().equals(uuid))
        return item;
    }
    return null; 
  }
  
  /**
   * @param jcrPath
   * @return current (Note: there can be multiple items with the same path) 
   * item by path or null if there are no one in session   
   * @throws RepositoryException
   */
  synchronized ItemImpl get(JCRPath jcrPath) {
    ItemImpl item = null;
    for(Iterator it=itemChanges.iterator(); it.hasNext();) {
      ItemImpl curItem = (ItemImpl)it.next();
      if (curItem.getLocation().equals(jcrPath))
        item = curItem;
    }
    return item;
  }

  synchronized List<ItemImpl> getChanges(JCRPath jcrPath, boolean childrenOnly) throws RepositoryException {

    List<ItemImpl> items = new ArrayList<ItemImpl>();
    for(Iterator it=itemChanges.iterator(); it.hasNext();) {
      ItemImpl curItem = (ItemImpl)it.next();
      JCRPath testLocation = curItem.getLocation();
      if ((testLocation.equals(jcrPath) || testLocation.isDescendantOf(jcrPath, childrenOnly))) {
        items.add(curItem);
      }
    }
    return items;
  }
  
  synchronized List getChanges() {
    return itemChanges;
  }
  
}
