/*
 * Copyright 2005 Day Management AG, Switzerland. All rights reserved.
 */
package org.exoplatform.services.jcr.impl.core.value;

import java.io.UnsupportedEncodingException;

import javax.jcr.PropertyType;
import javax.jcr.ValueFormatException;

import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.dataflow.TransientValueData;

/**
 * A <code>StringValue</code> provides an implementation
 * of the <code>Value</code> interface representing a string value.
 *
 * @author Stefan Guggisberg
 */
public class StringValue extends BaseValue {

    public static final int TYPE = PropertyType.STRING;

    protected final String text;

    /**
     * Constructs a <code>StringValue</code> object representing a string.
     *
     * @param text the string this <code>StringValue</code> should represent
     */
    public StringValue(String text) {
        super(TYPE, new TransientValueData(text));
        this.text = text;
    }

    StringValue(ValueData data) {
        super(TYPE, data);
        try {
           this.text = new String(data.getAsByteArray(), DEFAULT_ENCODING);
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException("FATAL ERROR Charset "+BaseValue.DEFAULT_ENCODING+" is not supported!");
		}   

    }

    /**
     * Indicates whether some other object is "equal to" this one.
     * <p>
     * The result is <code>true</code> if and only if the argument is not
     * <code>null</code> and is a <code>StringValue</code> object that
     * represents the same value as this object.
     *
     * @param   obj   the reference object with which to compare.
     * @return  <code>true</code> if this object is the same as the obj
     *          argument; <code>false</code> otherwise.
     */
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj instanceof StringValue) {
            StringValue other = (StringValue) obj;
            if (text == other.text) {
                return true;
            } else if (text != null && other.text != null) {
                return text.equals(other.text);
            }
        }
        return false;
    }

    /**
     * @see BaseValue#getInternalString()
     */
    protected String getInternalString() throws ValueFormatException {
	if (text != null) {
	    return text;
	} else {
	    throw new ValueFormatException("empty value");
	}
    }
}
