/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.dataflow;

import java.io.UnsupportedEncodingException;
import java.util.Calendar;

import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.Uuid;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.core.value.BaseValue;
import org.exoplatform.services.jcr.impl.core.value.DateValue;

/**
 * Created by The eXo Platform SARL        .<br/>
 * 
 * Newly created value data
 * TODO avoid using BaseValue and  DateValue here
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TransientValueData.java 5375 2006-05-03 15:34:12Z peterit $
 */

public class TransientValueData implements ValueData {

  private byte[] data;

  private int orderNum = 0;

  /**
   * Constructor for stored value data
   * @param value
   */
  public TransientValueData(byte[] value) {
    this.data = value;
  }

  /**
   * Constructor for String value data
   * @param value
   */
  public TransientValueData(String value) {
    try {
      this.data = value.getBytes(BaseValue.DEFAULT_ENCODING);
    } catch (UnsupportedEncodingException e) {
      throw new RuntimeException("FATAL ERROR Charset "
          + BaseValue.DEFAULT_ENCODING + " is not supported!");
    }
  }

  /**
   * Constructor for boolean value data
   * @param value
   */
  public TransientValueData(boolean value) {
    this.data = Boolean.valueOf(value).toString().getBytes();
  }

  /**
   * Constructor for Calendar value data
   * @param value
   */
  public TransientValueData(Calendar value) {
    //this.data = ISO8601.format(value).getBytes();
    this.data = DateValue.serializeDate(value);
  }

  /**
   * Constructor for double value data
   * @param value
   */
  public TransientValueData(double value) {
    this.data = Double.valueOf(value).toString().getBytes();
  }
  
  /**
   * Constructor for long value data
   * @param value
   */
  public TransientValueData(long value) {
    this.data = Long.valueOf(value).toString().getBytes();
  }

  /**
   * Constructor for Name value data
   * @param value
   */
  public TransientValueData(InternalQName value) {
    this.data = value.getAsString().getBytes();
  }

  /**
   * Constructor for Path value data
   * @param value
   */
  public TransientValueData(InternalQPath value) {
    this.data = value.getAsString().getBytes();
  }

  /**
   * Constructor for Reference value data
   * @param value
   */
  public TransientValueData(Uuid value) {
    this.data = value.getString().getBytes();
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.ValueData#getAsByteArray()
   */
  public byte[] getAsByteArray() {
    return data;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.ValueData#getOrderNumber()
   */
  public int getOrderNumber() {
    return orderNum;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.datamodel.ValueData#setOrderNumber(int)
   */
  public void setOrderNumber(int orderNum) {
    this.orderNum = orderNum;
  }

}