/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.dataflow.data;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.dataflow.persistent.PersistedItemData;
import org.exoplatform.services.jcr.dataflow.persistent.PersistedNodeData;
import org.exoplatform.services.jcr.dataflow.persistent.PersistedPropertyData;
import org.exoplatform.services.jcr.dataflow.persistent.PersistedValueData;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.datamodel.ValueData;

/**
 * Created by The eXo Platform SARL 
 * 
 * @author <a href="mailto:peter.nedonosko@exoplatform.com.ua">Peter Nedonosko</a>
 */
public class ItemDataFactory {

  public static PersistedItemData makeItemData(ItemData itemData) throws RepositoryException {
    if (itemData instanceof NodeData) {
      return makeNodeData((NodeData) itemData);
    } else if (itemData instanceof PropertyData) {
      return makePropertyData((PropertyData) itemData);
    } else {
      return null;
    }
  }

  public static PersistedNodeData makeNodeData(NodeData nodeData) throws RepositoryException { 

    String uuid = nodeData.getUUID();
    if (uuid == null) {
      throw new RepositoryException("Node data has no UUID: "
          + (nodeData.getQPath() != null ? nodeData.getQPath().getAsString() : 
            "[node data has no path]"));
    }
    
    InternalQPath qpath = nodeData.getQPath();
    if (qpath == null) {
      throw new RepositoryException("Node data has no path: " 
          + nodeData.getUUID());
    }
    
    InternalQName ptn = nodeData.getPrimaryTypeName();
    
    /*
     String id, InternalQPath qpath, String parentId, int version,
      int orderNumber, InternalQName primaryTypeName, InternalQName[] mixinTypeNames,
      AccessControlList acl*/
    PersistedNodeData rdbData = new PersistedNodeData(
        new String(uuid),
        qpath,
        nodeData.getParentUUID() != null ? new String(nodeData.getParentUUID()) : null,
        nodeData.getPersistedVersion(),
        nodeData.getOrderNumber(),
        ptn,
        nodeData.getMixinTypeNames(),
        nodeData.getChildrenACL()
        ); 

    return rdbData;
  }

  public static PersistedPropertyData makePropertyData(PropertyData propertyData) throws RepositoryException { 

    String uuid = propertyData.getUUID();
    if (uuid == null) {
      throw new RepositoryException("Property data has no UUID: "
          + (propertyData.getQPath() != null ? propertyData.getQPath().getAsString() : 
            "[property data has no path]"));
    }
    
    InternalQPath qpath = propertyData.getQPath();
    if (qpath == null) {
      throw new RepositoryException("Property data has no path: " 
          + propertyData.getUUID());
    }
    
    int pt = propertyData.getType();
    if (pt < 0) { //UNDEFINED=0
      throw new RepositoryException("Property data has no valid type: "  + pt + " "
          + propertyData.getQPath().getAsString() + " " + propertyData.getUUID());
    }

    List<ValueData> values = new ArrayList<ValueData>();
    for (Object ov : propertyData.getValues()) {
      if (ov instanceof ValueData) {
        values.add(makeValueData((ValueData) ov));
      }
    }    
    
    PersistedPropertyData rdbData = new PersistedPropertyData(
        new String(uuid),
        qpath,
        propertyData.getParentUUID() != null ? new String(propertyData.getParentUUID()) : null,
        propertyData.getPersistedVersion(),    
        pt,
        propertyData.isMultiValued(),
        values,
        null // TODO temporary
        );
    
    return rdbData;
  }

  public static PersistedValueData makeValueData(ValueData valueData) {

    PersistedValueData rdbData = new PersistedValueData(
        valueData.getAsByteArray(),
        valueData.getOrderNumber()
        );
    return rdbData;
  }
  
  public static List<NodeData> makeCopyOfNodes(List childNodes) throws RepositoryException {
    List<NodeData> copyOfChildsNodes = new LinkedList<NodeData>();
    for (Object nodeData: childNodes) {
      if (nodeData instanceof NodeData) {
        NodeData copyOfItem = makeNodeData( (NodeData) nodeData);
        if (copyOfItem != null) {
          copyOfChildsNodes.add(copyOfItem);
        }
      } // else ignore, can check it after by size comparing
    }
    return copyOfChildsNodes;
  }
  
  public static List<PropertyData> makeCopyOfProperties(List traverseProperties) throws RepositoryException {
    List<PropertyData> copyOfChildsProperties = new LinkedList<PropertyData>();
    for (Object nodeProperty: traverseProperties) {
      if (nodeProperty instanceof PropertyData) {
        PropertyData copyOfItem = makePropertyData( (PropertyData) nodeProperty);
        if (copyOfItem != null) {
          copyOfChildsProperties.add(copyOfItem);
        } 
      } // else ignore, can check it after by size comparing
    }
    return copyOfChildsProperties;
  }  
}
