/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.dataflow.session;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.impl.Constants;
import org.exoplatform.services.jcr.impl.core.ItemImpl;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.core.version.VersionHistoryImpl;
import org.exoplatform.services.jcr.impl.core.version.VersionImpl;

/**
 * Created by The eXo Platform SARL        .
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: ItemFactory.java 5800 2006-05-28 18:03:31Z geaz $
 * 
 */
class ItemFactory {

  private final SessionImpl session;
  
  public ItemFactory(SessionImpl session) {
    this.session = session;
  }

  public ItemImpl createItem(ItemData data) throws RepositoryException {
    if (data.isNode())
      return createNode(data);
    else
      return createProperty(data);
  }

  public NodeImpl createNode(ItemData data) throws RepositoryException {
    NodeImpl node = new NodeImpl(data.getUUID(), session);
    if (node.isNodeType(Constants.NT_VERSION)) {
      return new VersionImpl(data.getUUID(), session);
    } else if (node.isNodeType(Constants.NT_VERSION_HISTORY))
      return new VersionHistoryImpl(data.getUUID(), session);
    else
      return node;
  }
  
  public PropertyImpl createProperty(ItemData data) {
    return new PropertyImpl(data.getUUID(), session);
  }

}
