/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.jcr.impl.dataflow.session;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.AccessDeniedException;
import javax.jcr.ReferentialIntegrityException;
import javax.jcr.RepositoryException;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;

import org.apache.commons.logging.Log;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.impl.core.ItemChangeState;
import org.exoplatform.services.jcr.impl.core.ItemImpl;
import org.exoplatform.services.jcr.impl.core.ItemState;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.PropertyImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.jcr.impl.dataflow.DataManager;
import org.exoplatform.services.jcr.impl.dataflow.ItemDataChangesLog;
import org.exoplatform.services.jcr.impl.dataflow.TransientItemData;
import org.exoplatform.services.jcr.impl.dataflow.TransientNodeData;
import org.exoplatform.services.jcr.impl.dataflow.TransientPropertyData;
import org.exoplatform.services.jcr.impl.dataflow.persistent.BatcheableWorkspaceDataManager;
import org.exoplatform.services.log.ExoLogger;

/**
 * Created by The eXo Platform SARL        .
 *
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TransactionableDataManager.java 5171 2006-04-18 08:17:58Z peterit $
 */
public class TransactionableDataManager implements DataManager {

  private BatcheableWorkspaceDataManager storageDataManager;
  
  private SessionImpl session;
  
  protected static Log log = ExoLogger.getLogger("jcr.TransactionableDataManager");
  
  //private TransactionLog transactionLog;
  
  public TransactionableDataManager(BatcheableWorkspaceDataManager storageDataManager,
      SessionImpl session) {
    super();
    this.storageDataManager = storageDataManager;
    this.session = session;
  }

  // --------------- ItemDataConsumer --------

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getChildNodesData(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public List<NodeData> getChildNodesData(NodeData parent) throws RepositoryException {
    return storageDataManager.getChildNodesData(parent);
  }


  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getChildPropertiesData(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public List<PropertyData> getChildPropertiesData(NodeData parent) throws RepositoryException {
    return storageDataManager.getChildPropertiesData(parent);
  }


  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getItemData(org.exoplatform.services.jcr.datamodel.InternalQPath)
   */
  public ItemData getItemData(InternalQPath path) throws RepositoryException {
    return storageDataManager.getItemData(path);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getItemData(java.lang.String)
   */
  public ItemData getItemData(String uuid) throws RepositoryException {
    return storageDataManager.getItemData(uuid);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getReferencesData(java.lang.String)
   */
  public List<PropertyData> getReferencesData(String uuid) throws RepositoryException {
    return storageDataManager.getReferencesData(uuid);
  }
  
  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getChildNodesCount(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public int getChildNodesCount(NodeData parent) throws RepositoryException {
    return storageDataManager.getChildNodesCount(parent);
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.dataflow.ItemDataConsumer#getChildPropertiesCount(org.exoplatform.services.jcr.datamodel.NodeData)
   */
  public int getChildPropertiesCount(NodeData parent) throws RepositoryException {
    return storageDataManager.getChildPropertiesCount(parent);
  }



  // ---------------  --------

  /**
   * Mark begin of transaction
   */
  public void beginTransaction() {
  }

  
  public void commit() {
  }
 
  public void rollback() {
    
  }

  /**
   * Updates the manager with new changes
   * If transaction is started it will fill manager's changes log,
   * else just move changes to workspace storage manager
   * It saves the changes AS IS - i.e. id DOES NOT care about 
   * cloning of this objects etc. 
   * @param changes
   * @throws RepositoryException
   */
  public void save(ItemDataChangesLog changes) throws RepositoryException  {
    storageDataManager.save(changes);
  }

  /**
   * Temporary method, will be removed after Session refactoring
   * It MAKES copy of inputting data 
   * For newly ADDED nodes it will also traverse children and add
   * them to output(save or pass) 
   * @param item
   */
   public void saveItem(ItemImpl item) throws RepositoryException {

     validate(item);

     ItemDataChangesLog changeLog;
     if (item.isNode() && item.isNew()) { // Traverse ADDED nodes and pass to
       // update
       List<ItemState> changes = new ArrayList<ItemState>();
       traverseAddedNode(item.getActualItemData(), changes);
       changeLog = new ItemDataChangesLog(changes, session);
     } else {
       TransientItemData newData = copyItemData(item.getActualItemData());
       ItemState state = new ItemState(newData, item.getState());
       changeLog = new ItemDataChangesLog(state, session);
       log.debug("save: " + state.getData().getQPath().getAsString() + " "
          + ItemChangeState.getStateName(state.getState()));
     }
     save(changeLog);
  }

  protected void traverseAddedNode(TransientItemData item, List<ItemState> items)
      throws RepositoryException {
 
    items.add(new ItemState(copyItemData(item), ItemChangeState.ADDED));

    // traverse if node
    if (item.isNode()) {
      TransientNodeData node = (TransientNodeData)item;
      List props = node.getChildProperties();
      for (int i = 0; i < props.size(); i++) {
        traverseAddedNode((TransientPropertyData) props.get(i), items);
      }
      List nodes = node.getChildNodes();
      for (int i = 0; i < nodes.size(); i++) {
        traverseAddedNode((TransientNodeData) nodes.get(i), items);
      }
    } 
  }

  private TransientItemData copyItemData(TransientItemData item) {
    // make a copy
    if (item.isNode()) {

      TransientNodeData node = (TransientNodeData)item;
      return new TransientNodeData(node.getQPath(), node.getUUID(), 
        node.getPersistedVersion(), node.getPrimaryTypeName(), node.getMixinTypeNames(),
        node.getOrderNumber(), node.getParentUUID(), node.getOrderDirection());
    } else {
      
      TransientPropertyData prop = (TransientPropertyData)item;
      // make a copy
      TransientPropertyData newData = new TransientPropertyData(
          prop.getQPath(), prop.getUUID(), prop.getPersistedVersion(),
          prop.getType(), prop.getParentUUID(), prop.isMultiValued());
      newData.setValues(prop.getValues());
      return newData;
    }
  }
  
  private void validate(ItemImpl item) throws RepositoryException,
      AccessDeniedException, ReferentialIntegrityException, LockException,
      ConstraintViolationException {

    // SessionImpl session = (SessionImpl) item.getSession();

    // Check permissions
    // delete item
    if (item.getState() == ItemChangeState.DELETED) {

      // check for manadatory property
      if (!item.isNode()) {

        PropertyImpl prop = (PropertyImpl) item;
        if (prop.getDefinition().isMandatory()) {
          NodeImpl itemParent = (NodeImpl) item.getParent();
          // Check if parent will be removed
          if (itemParent.isRoot()
              || itemParent.getState() != ItemChangeState.DELETED) {
            throw new ConstraintViolationException(
                "Can not remove mandatory property " + prop.getPath());
          } // else perent node will be removed too
        }
      }

      if (!item.hasPermission(PermissionType.REMOVE))
        throw new AccessDeniedException("Access denied for REMOVE "
            + item.getPath());

      // [PN] 31.01.06
      // looking at actual item data in persistent storage
//      if (item.getPersistedVersion() > 0) {
//        ItemData itemData = getItemData(item.getLocation());
//        if (itemData == null) {
//          String message = "The changes to be persisted conflicts with a change already persisted through another session. "
//              + "Removing item already deleted by other session. Item: ";
//          throw new InvalidItemStateException(message + item.getPath());
//        }
//      }
    }

    // add node
    if (item.isNode() && item.getState() == ItemChangeState.ADDED) {

      // check for manadatory property
      if (item.isNode()) {
        NodeImpl node = (NodeImpl) item;
        node.validateMandatoryChildren();
      }

      if (!item.hasPermission(PermissionType.ADD_NODE))
        throw new AccessDeniedException("Access denied for ADD "
            + item.getPath());

    }

    // add/update property
    if (!item.isNode()
        && (item.getState() == ItemChangeState.UPDATED || item.getState() == ItemChangeState.ADDED)) {

      if (!item.hasPermission(PermissionType.SET_PROPERTY))
        throw new AccessDeniedException("Access denied for ADD "
            + item.getPath());

    }
  }

}
