/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.storage.rdb.container.data;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.access.AccessControlList;
import org.exoplatform.services.jcr.dataflow.persistent.PersistedNodeData;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: MultiRDBNodeRecord.java 5330 2006-04-28 16:00:52Z peterit $
 * 
 * @hibernate.joined-subclass table="JCR_MNODE" dynamic-update="true" l--azy="true"
 * @hibernate.joined-subclass-key column="ID"
 * @hibernate.collection-index column="ORDER_NUM"
 * @hibe---rnate.cache usage="read-write"
 */

public class MultiRDBNodeRecord extends MultiRDBItemRecord implements NodeRecord {

  protected int orderNum;

  protected Set properties = new HashSet();

  protected Set nodes = new HashSet();

  protected InternalQName primaryType;

  protected InternalQName[] mixinTypes;

  protected AccessControlList acl;

  public MultiRDBNodeRecord() {
  }

  private void toLog(String m) {
    //System.out.println("NodeRecord: " + path + " " + m);
  }
   
//  /**
//   * @h--ibernate.many-to-one column="ID"
//   *    class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBItemRecord"
//   *    foreign-key="JCR_FK_NODE_ITEM"
//   */
//  public String getId() {
//    return super.getId();
//  }
//
//  public void setId(String id) {
//    super.setId(id);
//  }

  /**
   * --lazy="false"--
   * 
   * @hibernate.set cascade="all" inverse="true" l--azy="true"
   * 
   * @hibernate.collection-one-to-many class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBPropertyRecord"
   * @hibernate.collection-key column="PARENT_ID"
   *  
   */
  public Set getProperties() {
    //toLog("getProperties(): " + (properties != null ? properties.size() : "null"));
    return properties;
  }

  public List getChildProperties() {
    ArrayList props = null;
    if (properties != null)
      props = new ArrayList(properties);
    return props;

  }

  /**
   * @param properties
   */
  public void setProperties(Set properties) {
    //toLog("setProperties(): " + (properties != null ? properties.size() : "null"));
    this.properties = properties;
  }

  public void addProperty(PropertyRecord property) {
    property.setParentNode(this);
    if (properties == null)
      properties = new HashSet();
    this.properties.add(property);
  }

  /**
   * ---lazy="proxy"---
   * 
   * @hibernate.set cascade="all" inverse="true" l--azy="true"
   * 
   * @hibernate.collection-one-to-many class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBNodeRecord"
   * @hibernate.collection-key column="PARENT_ID"
   */
  public Set getNodes() {
    //toLog("getNodes(): " + (nodes != null ? nodes.size() : "null"));
    return nodes;
  }

  public void setNodes(Set childNodes) {
    //toLog("setNodes(): " + (childNodes != null ? childNodes.size() : "null"));
    this.nodes = childNodes;
    //this.nodes = new HashSet(); // [PN] 31.03.06 Experimental
  }

  public void addChildNode(NodeRecord node) {
    node.setParentNode(this);
    if (nodes == null)
      nodes = new HashSet();
    this.nodes.add(node);
  }

  public List getChildNodes() {
    ArrayList nodeList = null;
    if (nodes != null) {
      nodeList = new ArrayList(nodes);
    }
    return nodeList;
  }

  public InternalQName getPrimaryTypeName() {
    return primaryType;
  }

  public AccessControlList getChildrenACL() {
    return acl;
  }

  public InternalQName[] getMixinTypeNames() {
    return mixinTypes;
  }

  public void setPrimaryType(InternalQName primaryType) {
    this.primaryType = primaryType;
  }

  public void setMixinTypes(InternalQName[] mixinTypes) {
    this.mixinTypes = mixinTypes;
  }

  public void setChildrenACL(AccessControlList acl) {
    this.acl = acl;
  }

  /**
   * @hibernate.property column="ORDER_NUM"
   */
  public int getOrderNumber() {
    //toLog("==== getOrderNumber(): " + orderNum);
    return orderNum;
  }

  public void setOrderNumber(int orderNum) {
    this.orderNum = orderNum;
  }

  /**
   * @hibernate.many-to-one column="PARENT_ID"
   *                        class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.MultiRDBNodeRecord"
   *                        foreign-key="JCR_FK_NODEPARENT"
   */
  public NodeRecord getParentNode() {    
    //toLog("==== NODE getParentNode(): " + (parentNode != null ? parentNode.getUUID() + "  " + parentNode.getPath() : "[null]") );
    return parentNode;
  }

  public void setParentNode(NodeRecord parent) {
    this.parentNode = parent;
    if(parent != null)
      this.parentUUID = new String(parent.getId());
  }
  	
  public boolean isNode() {
    return true;
  }

  public boolean isOrdered() {
    // For persisted node data - not ordered always 
    return getOrderDirection() != 0;
  }

  public int getOrderDirection() {
    return 0;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.data.ItemRecord#makeCopy()
   */
  public ItemData makeCopy() throws RepositoryException {
    return new PersistedNodeData(id, InternalQPath.parse(path), parentNode.getId(), version,
        orderNum, primaryType, mixinTypes,  acl);
    //return null;
  }
}