/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.rdb.container.data;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.jcr.RepositoryException;

import org.exoplatform.services.jcr.access.AccessControlList;
import org.exoplatform.services.jcr.dataflow.persistent.PersistedNodeData;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.impl.core.UUIDGenerator;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: SingleRDBNodeRecord.java 5330 2006-04-28 16:00:52Z peterit $
 * 
 * @hibernate.joined-subclass table="JCR_SNODE" dynamic-update="true"
 * @hibernate.joined-subclass-key column="ID"
 * @hibernate.collection-index column="ORDER_NUM"
 *  
 */
public class SingleRDBNodeRecord extends SingleRDBItemRecord implements
    NodeRecord {

  protected int orderNum;

  protected Set properties = new HashSet();

  protected Set nodes = new HashSet();

  protected InternalQName primaryType;

  protected InternalQName[] mixinTypes;

  protected AccessControlList acl;

  public SingleRDBNodeRecord() {
  }

  /**
   * @hibernate.set cascade="all" inverse="true"
   * 
   * @hibernate.collection-one-to-many class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.SingleRDBPropertyRecord"
   * @hibernate.collection-key column="PARENT_ID"
   *  
   */
  public Set getProperties() {
    return properties;
  }

  public List getChildProperties() {
    ArrayList props = null;
    if (properties != null)
      props = new ArrayList(properties);
    return props;

  }

  /**
   * @param properties
   */
  public void setProperties(Set properties) {
    this.properties = properties;
  }

  public void addProperty(PropertyRecord property) {
    property.setParentNode(this);
    if (properties == null)
      properties = new HashSet();
    this.properties.add(property);
  }

  /**
   * ---lazy="proxy"---
   * 
   * @hibernate.set cascade="all" inverse="true"
   * 
   * @hibernate.collection-one-to-many class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.SingleRDBNodeRecord"
   * @hibernate.collection-key column="PARENT_ID"
   */
  public Set getNodes() {
    return nodes;
  }

  public void setNodes(Set childNodes) {
    this.nodes = childNodes;
  }

  public void addChildNode(NodeRecord node) {
    node.setParentNode(this);
    if (nodes == null)
      nodes = new HashSet();
    this.nodes.add(node);
  }

  public List getChildNodes() {
    ArrayList nodeList = null;
    if (nodes != null)
      nodeList = new ArrayList(nodes);

    return nodeList;
  }

  public InternalQName getPrimaryTypeName() {
    return primaryType;
  }

  public AccessControlList getChildrenACL() {
    return acl;
  }

  public InternalQName[] getMixinTypeNames() {
    return mixinTypes;
  }

  public void setPrimaryType(InternalQName primaryType) {
    this.primaryType = primaryType;
  }

  public void setMixinTypes(InternalQName[] mixinTypes) {
    this.mixinTypes = mixinTypes;
  }

  public void setChildrenACL(AccessControlList acl) {
    this.acl = acl;
  }

  public boolean isNode() {
    return true;
  }

  /**
   * @hibernate.property column="ORDER_NUM"
   */
  public int getOrderNumber() {
    return orderNum;
  }

  public void setOrderNumber(int orderNum) {
    this.orderNum = orderNum;
  }

  /**
   * @hibernate.many-to-one column="PARENT_ID"
   *                        class="org.exoplatform.services.jcr.impl.storage.rdb.container.data.SingleRDBNodeRecord"
   *                        foreign-key="JCR_FK_NODEPARENT"
   */
  public NodeRecord getParentNode() {
    return parentNode;
  }

  public void setParentNode(NodeRecord parent) {
    this.parentNode = parent;
    if(parent != null) {
      int start = parent.getId().length() - UUIDGenerator.UUID_LENGTH; 
      this.parentUUID = new String(parent.getId().substring(start, parent.getId().length()));

    }
  }
  
  public boolean isOrdered() {
    // For persisted node data - not ordered always 
    return getOrderDirection() != 0;
  }

  public int getOrderDirection() {
    return 0;
  }

  /* (non-Javadoc)
   * @see org.exoplatform.services.jcr.impl.storage.rdb.container.data.ItemRecord#makeCopy()
   */
  public ItemData makeCopy() throws RepositoryException {
    return new PersistedNodeData(id, InternalQPath.parse(path), parentNode.getId(), version,
        orderNum, primaryType, mixinTypes,  acl);
    //return null;
  }

}